<?php
namespace Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class Mym00112UsuariosQrCodeController extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $connection;
    protected $dataSourceManager;
    protected $environment;
    protected $FCMService;
    protected $qrCodeService;

    public function __construct($crudService, $parameterBag, $connection, $dataSourceManager, $environment, $FCMService, $qrCodeService){
        parent::__construct($crudService, $parameterBag);
        $this->connection        = $connection;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment       = $environment;
    	$this->FCMService        = $FCMService;
    	$this->qrCodeService     = $qrCodeService;
    }
    
    public function find($request, $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $nrlista = "";
        foreach($conditions as $key => &$condition) {
            if($condition['columnName'] == 'CDFILIAL' && $condition['value'] != 'T') {
                $nrlista = $this->criarListaValores($condition['value']);
                $condition['value'] = $nrlista;
            }
            if($condition['columnName'] == 'NMFILIAL') {
                unset($conditions[$key]);
            }
        }
        array_push($conditions, array(
                "columnName"=> "CDOPERADOR",
                "value"=> $this->environment->getCdOperador(),
                "operator"=> "=",
            )
        );
        $dataSourceName = 'qrcodeusers';
        
        $result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions, $page, $pageSize)->getRows();
        $response->addDataSet(new DataSet('qrcodeusers',$result));
        $this->apagarListaValores($nrlista);
    }

    public function atualizaUsuario($request, $response) {
        $row = $request->getRow();
        $this->connection->beginTransaction();
        try {
            // Verifica se existe o mesmo identificador unico em outro usuario.
            $queryUserID = "SELECT COUNT(CDREGISTROPESSOA) AS QTDE FROM USUARIOAPP WHERE CDREGISTROPESSOA = :CDREGISTROPESSOA AND NRUSUARIOAPP <> :NRUSUARIOAPP";
            $queryParams = [
                "CDREGISTROPESSOA"   => $row["CDREGISTROPESSOA"],
                "NRUSUARIOAPP"       => $row["NRUSUARIOAPP"]
    	    ];
    		
    		$statementQuery = $this->connection->prepare($queryUserID);
    		$statementQuery->execute($queryParams);
            $countResult = $statementQuery->fetch();
    		
    		if($countResult['QTDE']!=0){
    		    throw new \Exception("Operação bloqueada! O ID já existe.");
    		}
            
            $queryUsuario = "UPDATE USUARIOAPP SET DSPRIMEIRONOME = :DSPRIMEIRONOME, DSULTIMONOME = :DSULTIMONOME, CDREGISTROPESSOA = :CDREGISTROPESSOA WHERE NRUSUARIOAPP = :NRUSUARIOAPP";
            $queryParams = [
                "NRORG"              => $row["NRORG"],
                "DSPRIMEIRONOME"     => $row["DSPRIMEIRONOME"],
                "DSULTIMONOME"       => $row["DSULTIMONOME"],
                "CDREGISTROPESSOA"   => $row["CDREGISTROPESSOA"],
                "NRUSUARIOAPP"       => $row["NRUSUARIOAPP"]
    	    ];
    		$statementQuery = $this->connection->prepare($queryUsuario);
    		$statementQuery->execute($queryParams);
            $queryIdAtivo = "UPDATE APP_USUARIOAPLICATIVO SET IDATIVO = :IDATIVO WHERE NRUSUARIOAPLICATIVO = :NRUSUARIOAPLICATIVO";
            $queryParams = [
                "NRORG"             => $row["NRORG"],
                "IDATIVO"             => $row["IDATIVO"],
                "NRUSUARIOAPLICATIVO" => $row["NRUSUARIOAPLICATIVO"]
    		];
    		
    		$statementQuery = $this->connection->prepare($queryIdAtivo);
    		$statementQuery->execute($queryParams);
            
            $this->connection->commit();
    	    if($row["IDATIVO"] == "N") {
        		$topicName = $this->qrCodeService->getTopicName($row["NRQRCODELOCALPESQ"]);
        		$this->FCMService->sendMessage($topicName, ["TYPE" => "MSG_DEACTIVATED_USER", "NRUSUARIOAPP" => $row["NRUSUARIOAPP"]]);
    	    }
        } catch(\Exception $e) {
            $this->connection->rollBack();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    //insere lista no banco
    private function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista,
            "NRORG" => $this->environment->getNrorgTrab()
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG) VALUES (:NRLISTA, :CDLISTA, :NRORG)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    //apaga lista do banco
    private function deleteLista($nrlista) {
        $params = array(
            "NRLISTA" => $nrlista
        );
        
        $sql = "DELETE FROM LISTA WHERE NRLISTA =:NRLISTA";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    //cria lista de valores recebendo string como parametro
    private function criarListaValores($valores){
        if(strlen($valores) <= 4000){
            return $valores;
        }else{
            $lista = explode('_',$valores);
            $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
            try{
                foreach($lista as $key){
                    $this->insertLista($nrlista, $key);
                }
                return "_NRLISTA=".$nrlista;
            } catch(\Exception $e){
                throw $e;
            }
        }
    }
    
    //apaga lista de valores recebendo string como parametro
    private function apagarListaValores($valores){
        try{
            if(substr($valores, 0,9) == "_NRLISTA="){
                $nrlista = substr($valores, 9);
                $this->deleteLista($nrlista);
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
}
